from __future__ import print_function

import argparse
import logging
import sys

from gcs_pypi import __prog__, __version__
from gcs_pypi.exceptions import gcspypiError
from gcs_pypi.package import Package
from gcs_pypi.storage import GCSStorage

log = logging.getLogger()


def create_and_upload_package(args):
    package = Package.create(args.wheel, args.sdist, args.dist_path)
    storage = GCSStorage(
        args.bucket, args.secret, args.bare, args.private, args.profile
    )

    index = storage.get_index(package)
    index.add_package(package, args.force)

    storage.put_package(package, args.dist_path)
    storage.put_index(package, index)


def parse_args(args):
    p = argparse.ArgumentParser(prog=__prog__)
    p.add_argument("--bucket", required=True, help="GCS bucket")
    p.add_argument("--secret", help="GCS secret")
    p.add_argument("--profile", help="AWS profile")
    p.add_argument("--force", action="store_true", help="Overwrite existing packages")
    p.add_argument(
        "--no-wheel", dest="wheel", action="store_false", help="Skip wheel distribution"
    )
    p.add_argument(
        "--no-sdist", dest="sdist", action="store_false", help="Skip sdist distribution"
    )
    p.add_argument(
        "--dist-path", help="Path to directory with wheel/sdist to be uploaded"
    )
    p.add_argument(
        "--bare", action="store_true", help="Store index as bare package name"
    )
    p.add_argument(
        "--private", action="store_true", help="Store GCS Keys as private objects"
    )
    p.add_argument("--verbose", action="store_true", help="Turn on verbose output.")
    p.add_argument("--version", action="version", version=__version__)
    return p.parse_args(args)


def main(*args):
    args = parse_args(args or sys.argv[1:])

    log.setLevel(logging.DEBUG if args.verbose else logging.INFO)

    try:
        create_and_upload_package(args)
    except gcspypiError as e:
        print("error: %s" % e)
        sys.exit(1)


if __name__ == "__main__":
    main()



import logging
import os
import re
import sys
from collections import defaultdict
from subprocess import CalledProcessError, check_output
from zipfile import ZipFile

from jinja2 import Environment, PackageLoader

from gcs_pypi import __prog__
from gcs_pypi.exceptions import gcspypiError

log = logging.getLogger()


class Package(object):
    """Python package."""

    def __init__(self, name, files):
        self.name, self.version = name.rsplit("-", 1)
        self.files = set(files)

    def __str__(self):
        return "%s-%s" % (self.name, self.version)

    def _attrs(self):
        return self.name, self.version

    def __lt__(self, other):
        return self.version < other.version

    def __eq__(self, other):
        return isinstance(other, Package) and self._attrs() == other._attrs()

    def __hash__(self):
        return hash(self._attrs())

    @property
    def directory(self):
        return re.sub(r"[-_.]+", "-", self.name.lower())

    @staticmethod
    def _find_wheel_name(text):
        match = re.search(
            r"creating '.*?(dist.*\.whl)' and adding", text, flags=re.MULTILINE
        )

        if not match:
            raise RuntimeError("Wheel name not found! (use --verbose to view output)")

        return match.group(1)

    @staticmethod
    def _find_name_from_wheel_metadata(text):
        name_match = re.search(r"^Name: (.*)", text, flags=re.MULTILINE)
        version_match = re.search(r"^Version: (.*)", text, flags=re.MULTILINE)

        if not name_match or not version_match:
            raise RuntimeError("Wheel name not found! (use --verbose to view output)")

        return "{}-{}".format(name_match.group(1), version_match.group(1))

    @staticmethod
    def create(wheel=True, sdist=True, dist_path=None):
        files = []
        if not dist_path:
            cmd = [sys.executable, "setup.py"]

            name = check_output(cmd + ["--fullname"]).decode().strip()

            if sdist:
                cmd.extend(["sdist", "--formats", "gztar"])

            if wheel:
                cmd.append("bdist_wheel")

            log.debug("Package create command line: {}".format(" ".join(cmd)))

            try:
                stdout = check_output(cmd).decode().strip()
            except CalledProcessError as e:
                raise RuntimeError(e.output.rstrip())

            log.debug(stdout)

            if sdist:
                files.append(name + ".tar.gz")

            if wheel:
                files.append(os.path.basename(Package._find_wheel_name(stdout)))
        else:
            for f in os.listdir(dist_path):
                if sdist and f.endswith(".tar.gz"):
                    name = f[:-7]
                if wheel and f.endswith(".whl"):
                    with ZipFile(os.path.join(dist_path, f), "r") as whl:
                        for fname in whl.namelist():
                            if fname.endswith("METADATA"):
                                name = Package._find_name_from_wheel_metadata(
                                    whl.open(fname).read().decode()
                                )

                files.append(f)

        log.debug("Package name: {}".format(name))
        log.debug("Files to upload: {}".format(files))

        return Package(name, files)


class Index(object):
    """Index of package versions, to be rendered to HTML."""

    template = Environment(loader=PackageLoader(__prog__, "templates")).get_template(
        "index.html.j2"
    )

    def __init__(self, packages):
        self.packages = set(packages)

    @staticmethod
    def parse(html):
        filenames = defaultdict(set)

        for match in re.findall(
            r'<a href=".+">((.+?-((?:\d+!)?\d+(?:\.\d+)*(?:(?:a|b|rc)\d+)?(?:\.post\d+)?'
            + r"(?:\.dev\d+)?)(?:\+([a-zA-Z0-9\.]+))?).*(?:\.whl|\.tar\.gz))</a>",
            html,
        ):
            filenames[match[1]].add(match[0])

        return Index(Package(name, files) for name, files in filenames.items())

    def to_html(self):
        return self.template.render({"packages": self.packages})

    def add_package(self, package, force=False):
        existing = next(
            (p for p in self.packages if p.version == package.version), None
        )
        if existing and force:
            self.packages.discard(existing)
            package = Package(str(package), existing.files | package.files)
        elif existing:
            raise gcspypiError(
                "%s already exists! You should use a different version (use --force to override)."
                % package
            )

        self.packages.add(package)



import logging
import os

from google.cloud import storage
from google.resumable_media import InvalidResponse

from gcs_pypi.package import Index

log = logging.getLogger()


class GCSStorage(object):
    """Abstraction for storing package archives and index files in a GCS bucket."""

    def __init__(self, bucket, secret=None, bare=False, private=False, profile=None):
        self.client = storage.Client()
        self.bucket = bucket
        self.secret = secret
        self.index = "" if bare else "index.html"
        self.acl = "private" if private else "publicRead"

    def _object(self, package, filename):
        path = "%s/%s" % (package.directory, filename)
        try:
            return self.client.get_bucket(self.bucket).get_blob(
                "%s/%s" % (self.secret, path) if self.secret else path
            )
        except AttributeError:
            return self.client.get_bucket(self.bucket).blob(
                "%s/%s" % (self.secret, path) if self.secret else path
            )

    def get_index(self, package):
        try:
            html = (
                self._object(package, self.index).download_as_string().decode("utf-8")
            )
            return Index.parse(html)
        except InvalidResponse:
            return Index([])

    def put_index(self, package, index):
        self._object(package, self.index).upload_from_string(
            data=index.to_html(), content_type="text/html", predefined_acl=self.acl,
        )

    def put_package(self, package, dist_path=None):
        for filename in package.files:
            path = os.path.join(dist_path or "dist", filename)
            log.debug("Uploading file `{}`...".format(path))
            with open(path, mode="rb") as f:
                self._object(package, filename).upload_from_file(
                    file_obj=f,
                    content_type="application/x-gzip",
                    predefined_acl=self.acl,
                )




# -*- coding: utf-8 -*-

# DO NOT EDIT THIS FILE!
# This file has been autogenerated by dephell <3
# https://github.com/dephell/dephell

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


import os.path

readme = ''
here = os.path.abspath(os.path.dirname(__file__))
readme_path = os.path.join(here, 'README.rst')
if os.path.exists(readme_path):
    with open(readme_path, 'rb') as stream:
        readme = stream.read().decode('utf8')


setup(
    long_description=readme,
    name='gcs_pypi',
    version='0.1.80',
    description='CLI for creating a Python Package Repository in a GCS bucket',
    python_requires='!=3.0.*,!=3.1.*,!=3.2.*,!=3.3.*,!=3.4.*,<4.0,>=2.7',
    author='Tonye Jack',
    author_email='jtonye@ymail.com',
    entry_points={"console_scripts": ["gcs_pypi = gcs_pypi.__main__:main"]},
    packages=['gcs_pypi'],
    package_dir={"": "."},
    package_data={"gcs_pypi": ["templates/*.j2"]},
    install_requires=['futures==3.*,>=3.3.0; python_version == "2.7.*" and python_version >= "2.7.0"', 'google-api-core==1.*,>=1.17.0', 'google-cloud-core==1.*,>=1.3.0', 'google-cloud-storage==1.*,>=1.28.0', 'jinja2==2.*,>=2.10.1', 'wheel==0.*,>=0.33.6'],
    extras_require={"dev": ["black==19.*,>=19.3.0.b0; python_version == \"3.*\" and python_version >= \"3.6.0\"", "bumpversion==0.5.3", "dephell==0.*,>=0.8.3; python_version == \"3.*\" and python_version >= \"3.6.0\"", "flake8==3.*,>=3.7.8", "ipdb==0.*,>=0.13.2", "isort==4.*,>=4.3.21", "mock==3.0.5", "pytest>=4.6.5", "pytest-cov==2.*,>=2.7.1"]},
)
